/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "endpointserver.h"

#include <conexus/utility.h>

namespace Conexus
{

  EndpointServer::EndpointServer()
      : Server()
  {}


  EndpointServer::~EndpointServer()
  {
    if ( m_running )
      stop();
  }

  void EndpointServer::start( )
  {
    this->start( use_dispatcher_default() );
  }

  void EndpointServer::start( bool use_dispatcher )
  {
    Server::start( use_dispatcher );

    if ( m_dispatcher && ! m_dispatcher_connection )
      m_dispatcher_connection = m_dispatcher->connect( sigc::mem_fun( *this, &EndpointServer::emit_endpoints ) );
  }

  void EndpointServer::stop()
  {
    if ( m_dispatcher_connection )
      m_dispatcher_connection.disconnect();

    Server::stop();
  }

  sigc::signal<void, Endpoint::pointer> EndpointServer::signal_new_endpoint()
  {
    return m_signal_new_endpoint;
  }

  void EndpointServer::queue_endpoint( Endpoint::pointer endpoint )
  {
    m_mutex_endpoint_queue.lock();
    m_endpoint_queue.push( endpoint );
    m_mutex_endpoint_queue.unlock();
    if ( m_mutex_signal_endpoint.trylock() ) {
      m_mutex_signal_endpoint.unlock();
      if ( m_use_dispatcher )
        m_dispatcher->emit();
      else
        emit_endpoints();
    }
  }

  void EndpointServer::emit_endpoints( )
  {
    Endpoint::pointer current;
    m_mutex_signal_endpoint.lock();
    m_mutex_endpoint_queue.lock();
    while ( ! m_endpoint_queue.empty() ) {
      current = m_endpoint_queue.front();
      m_endpoint_queue.pop();
      m_mutex_endpoint_queue.unlock();
      m_signal_new_endpoint.emit( current );
      m_mutex_endpoint_queue.lock();
    }
    m_mutex_endpoint_queue.unlock();
    m_mutex_signal_endpoint.unlock();
  }

}
