/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "pipe.h"

#include <unistd.h>

namespace Conexus
{

  Pipe::Pipe( ) :
      FileDescriptor(),
      m_write_fd( -1 )
  {}

  Pipe::pointer Pipe::create( )
  {
    return pointer( new Pipe( ) );
  }

  Pipe::~Pipe()
  {}

  void Pipe::open( ) throw( open_exception )
  {
    int fds[ 2 ];
    int result;

    result = pipe( fds );

    switch ( result ) {
      case 0:
        break;
      case EFAULT:
        break;
      case EMFILE:
        break;
      case ENFILE:
        break;
    }

    m_fd = fds[ 0 ];
    m_write_fd = fds[ 1 ];
    this->set_state_opened();
    m_state |= PIPE_READ | PIPE_WRITE;
  }

  void Pipe::close( bool force ) throw ( close_exception )
  {
    int i;
    if ( m_state & PIPE_READ ) {
      i = ::close( m_fd );
      if ( i == -1 )
        throw_close_exception( errno );
    }

    if ( m_state & PIPE_WRITE ) {
      i = ::close( m_write_fd );
      if ( i == -1 )
        throw_close_exception( errno );
    }

    this->set_state_closed();
  }

  void Pipe::close_read( bool force ) throw ( close_exception )
  {
    int i;
    if ( m_state & PIPE_READ ) {
      i = ::close( m_fd );
      if ( i == -1 )
        throw_close_exception( errno );
      m_state &= ~PIPE_READ;
    }
  }

  void Pipe::close_write( bool force ) throw ( close_exception )
  {
    int i;
    if ( m_state & PIPE_WRITE ) {
      i = ::close( m_write_fd );
      if ( i == -1 )
        throw_close_exception( errno );
      m_state &= ~PIPE_WRITE;
    }
  }

  ssize_t Pipe::write_data( long int to, const Data data ) throw ( write_exception )
  {
    int fds;
    ssize_t bytes;
    fd_set writefd;
    struct timeval timeout;

    if ( ! data )
      return 0;

    if ( this->is_closed() )
      this->open();

    if ( to == 0 ) {
      bytes = ::write( m_write_fd, data, data.size() );
      if ( bytes >= 0 )
        return bytes;
      else
        throw_write_exception( errno );
    } else {
      FD_ZERO( &writefd );
      FD_SET( m_write_fd, &writefd );
      timeout.tv_sec = 0;
      timeout.tv_usec = to;
      fds = ::select( m_write_fd + 1, NULL, &writefd, NULL, &timeout );
      if ( fds ) {
        bytes = ::write( m_write_fd, data, data.size() );
        if ( bytes >= 0 )
          return bytes;
        else
          throw_write_exception( errno );
      } else if ( fds == 0 ) {
        return 0;
      }
    }

    /* else fds < 0 */
    // TODO come back and put in proper select throws
    throw;
  }

  ssize_t Pipe::input_available()
  {
    return PIPE_BUF;
  }


  Data Pipe::read_data( long int timeout, size_t s ) throw ( read_exception )
  {
    Data d = FileDescriptor::read_data( timeout, s );

    if ( ! d )
      throw ( exception::read::disconnected() );

    return d;
  }

}

