/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "server.h"

#include <sys/select.h>

#include <conexus/utility.h>

namespace Conexus
{

  Server::Server() :
      m_thread( NULL ),
      m_running( false ),
      m_dispatcher( NULL ),
      m_use_dispatcher( false )
  {
    m_timeval.tv_sec = 0;
    m_timeval.tv_usec = 500000;
  }


  Server::~Server()
  {
    if ( m_running )
      stop();
  }

  void Server::start( )
  {
    this->start( use_dispatcher_default() );
  }

  void Server::start( bool use_dispatcher )
  {
    if ( m_running )
      return ;

    m_use_dispatcher = use_dispatcher;
    m_terminate = false;
    m_running = true;
    if ( m_use_dispatcher ) {
      m_dispatcher = new Glib::Dispatcher();
    }
    m_thread = Glib::Thread::create( sigc::mem_fun( *this, &Server::service_thread_main ), true );
  }

  void Server::stop()
  {
    m_terminate = true;
    m_thread->join();
    m_thread = NULL;
    m_running = false;
    if ( m_use_dispatcher ) {
      m_use_dispatcher = false;
      delete m_dispatcher;
      m_dispatcher = NULL;
    }
  }

  bool Server::is_running()
  {
    return m_running;
  }

  void Server::set_responsiveness( long r )
  {
    m_timeval.tv_usec = r;
  }

  long Server::responsiveness()
  {
    return m_timeval.tv_usec;
  }

  void Server::service_thread_main_proxy()
  {
    service_thread_main();
  }

}
